@@ -1,7 +1,7 @@
 # perl -w
 # ShowTable.pm
 #
-#    Copyright (C) 1996,1997  Alan K. Stebbens <aks@sgi.com>
+#    Copyright (C) 1996-2013 Alan K. Stebbens <aks@stebbens.org>
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
-# $Id: ShowTable.pm,v 1.20 1997/03/02 22:04:50 aks Exp $
-#
 
 package Data::ShowTable;
 
+$VERSION = '4.6';
+
 =head1 NAME
 
 B<ShowTable> - routines to display tabular data in several formats.
@@ -1283,8 +1283,8 @@ sub ShowTableValue {
 	    $d =~ s/(...)(?=.)/$1,/g;
 	    # reverse the digits and grouping char
 	    $d = '$'.join('',reverse(split(//,$d)));
-	    # If there is any precision, add on pennies
-	    $d .= sprintf(".%02d",$c) if $prec > 0;
+	    # If there is any precision, add on pennies (allow for > 2 precision)
+	    $d .= sprintf($prec > 2 ? "%0${prec}d" : ".%02d",$c) if $prec > 0;
 	    # Mark as negative with '(xxx)'
 	    $d = '-'.$d if $value < 0;
 	    $str = $d;
@@ -1304,7 +1304,7 @@ sub ShowTableValue {
     if ($width > length(&PlainText($str))) {
 	# right align the value if any kind of number
 	$str = sprintf("%${width}s", $str) 
-	    if $type =~ /int|float|real|numeric|money/i;
+	    if $type =~ /int|float|pct|real|numeric|money/i;
     }
     $str;
 }
@@ -1316,6 +1316,7 @@ sub ShowTableValue {
   'tinyint'	=> '%%%dd',
   'shortint'	=> '%%%dd',
   'int'		=> '%%%dd',
+  'pct'	        => '%%%d.%df%%%%',
   'real'	=> '%%%d.%df',
   'float'	=> '%%%d.%df',
   'numeric'	=> '%%%d.%df',
@@ -1325,8 +1326,8 @@ sub ShowTableValue {
   # ShowTableValue handle money formatting explicitly.  However, some
   # one else might use this table, so we treat them like right-aligned
   # strings.
-  'money'	=> '%%%ds',
-  'smallmoney'	=> '%%%ds',
+  'money'	=> '%%%d.%df',
+  'smallmoney'	=> '%%%d.%df',
 
   );
 
@@ -1673,8 +1674,12 @@ maximum default precision.
 
 is the ref to the array of maximum widths for the given columns.
 
+=back
+
 =head2 B<ARGUMENTS>
 
+=over 10
+
 =item I<$widthspec>
 
 A reference to an array of column width (or length) values, each given
@@ -1964,7 +1969,6 @@ sub putcell {
     my $prefix     = shift;	# 2-elt array of prefix strings
     my $suffix     = shift;	# 2-elt array of suffix strings
     my $wrap_flag  = shift;	# non-zero for wrapped lines
-    my $fmt        = sprintf("%%s%%-%ds%%s",$cell_width);
     my $more;
 
     my $v = $cells->[$c];	# get the data
@@ -1972,7 +1976,7 @@ sub putcell {
     my $sx = 0;			# suffix index
     if (defined $v) {		# not undef data?
 	my $text = $v;		# save the text
-	$cell_width = 1 unless $cell_width > 0;	# be sane
+        $cell_width = 1 if !defined($cell_width) || $cell_width == 0;
 	if ($cell_width <= length($text)) {
 	    $more = substr($text,$cell_width);
 	    $v = substr($text,0,$cell_width);
@@ -2005,6 +2009,7 @@ sub putcell {
 	$px = $wrap_flag != 0 && length($v) > 0;
 	$sx = length($more) > 0;
     }
+    my $fmt        = sprintf("%%s%%-%ds%%s",$cell_width);
     put $fmt,$prefix->[$px],$v,$suffix->[$sx];	# output something (could be blanks)
     $sx;			# leave wrapped flag
 }
@@ -2143,7 +2148,7 @@ sub put {
 
 =head1 AUTHOR
 
-Alan K. Stebbens <aks@sgi.com>
+Alan K. Stebbens <aks@stebbens.org>
 
 =cut
 
@@ -2157,6 +2162,8 @@ Embedded HTML is how the user can insert formatting overrides.  However,
 the HTML formatting techniques have not been given much consideration --
 feel free to provide constructive feedback.
 
+=back
+
 =cut
 
 #
